@@ -367,7 +367,10 @@ gtk_source_view_snippets_update_informative (GtkSourceViewSnippets *snippets)
 	}
 	else
 	{
-		gtk_widget_show (GTK_WIDGET (snippets->informative));
+		if (gtk_widget_get_mapped (GTK_WIDGET (snippets->view)))
+		{
+			gtk_widget_show (GTK_WIDGET (snippets->informative));
+		}
 	}
 
 	return;
@@ -565,7 +568,7 @@ _gtk_source_view_snippets_key_pressed (GtkSourceViewSnippets *snippets,
 	 * the view, such as with completion providers.
 	 */
 
-	if (snippets->buffer == NULL || snippets->queue.length == 0)
+	if (snippets->buffer == NULL)
 	{
 		return GDK_EVENT_PROPAGATE;
 	}
@@ -590,6 +593,8 @@ _gtk_source_view_snippets_key_pressed (GtkSourceViewSnippets *snippets,
 		 */
 		if (snippet != NULL)
 		{
+			_gtk_source_view_hide_completion (snippets->view);
+
 			if ((state & modifiers) == 0)
 			{
 				if (!_gtk_source_snippet_move_next (snippet))
@@ -626,6 +631,8 @@ _gtk_source_view_snippets_key_pressed (GtkSourceViewSnippets *snippets,
 		    !has_selection &&
 		    gtk_source_view_snippets_try_expand (snippets, &end))
 		{
+			_gtk_source_view_hide_completion (snippets->view);
+
 			gtk_source_view_snippets_scroll_to_insert (snippets);
 
 			ret = GDK_EVENT_STOP;
@@ -634,7 +641,7 @@ _gtk_source_view_snippets_key_pressed (GtkSourceViewSnippets *snippets,
 	}
 
 cleanup:
-	if (snippets->queue.length == 0)
+	if (ret != GDK_EVENT_PROPAGATE && snippets->queue.length == 0)
 	{
 		if (snippets->informative)
 		{
